"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const xml2js = require("xml2js");
const util = require("util");
const fs = require("fs-extra");
const metadata_json_1 = __importDefault(require("../../../metadata.json"));
const rimraf = require("rimraf");
const spawn = require('child-process-promise').spawn;
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages('sfpowerkit', 'valid');
class Valid extends command_1.SfdxCommand {
    async run() {
        // Getting Project config
        const project = await core_1.SfdxProject.resolve();
        const projectJson = await project.retrieveSfdxProjectJson();
        let packageToBeScanned = this.flags.package;
        this.ux.log("package:" + packageToBeScanned);
        const packageDirectories = projectJson.get('packageDirectories') || [];
        // this.ux.logJson(packageDirectories);
        const result_store = [];
        if (packageToBeScanned != undefined) {
            for (const sf_package of packageDirectories) {
                if (packageToBeScanned != undefined && packageToBeScanned === sf_package['package']) {
                    this.ux.log("Package to be analyzed located");
                    //this.ux.logJson(sf_package.valueOf());
                    let result;
                    try {
                        result = await this.validate(sf_package);
                        break;
                    }
                    catch (e) {
                        this.clearDirectory();
                        this.ux.log("Error Occured Unable to analyze");
                    }
                }
            }
        }
        else {
            this.ux.log("All packaging directories are  being analyzed");
            for (const sf_package of packageDirectories) {
                if (sf_package['package'] != undefined) {
                    this.ux.log(`Now analyzing ${sf_package['package']}`);
                    //this.ux.logJson(sf_package.valueOf());
                    let result;
                    try {
                        result = await this.validate(sf_package);
                    }
                    catch (e) {
                        this.clearDirectory();
                        this.ux.log("Error Occured Unable to analyze");
                    }
                    result_store.push(result);
                }
            }
        }
        result_store.forEach(element => {
            if (element == 1)
                throw new core_1.SfdxError("Analysis Failed, Unsupported metadata present");
        });
        return { message: 'Analyzing succesfully completed' };
    }
    async validate(packageToBeScanned) {
        // Split arguments to use spawn
        const args = [];
        args.push('force:source:convert');
        // outputdir
        args.push('-d');
        args.push('temp_sfpowerkit/mdapi');
        // package name
        args.push('-n');
        args.push(`${packageToBeScanned['package']}`);
        args.push('-r');
        args.push(`${packageToBeScanned['path']}`);
        // INSTALL PACKAGE
        this.ux.log(`Converting package ${packageToBeScanned['package']}`);
        var startTime = (new Date()).valueOf();
        await spawn('sfdx', args, { stdio: 'inherit' });
        let targetFilename = 'temp_sfpowerkit/mdapi/package.xml';
        if (fs.existsSync(targetFilename)) {
            const parser = new xml2js.Parser({ explicitArray: false });
            const parseString = util.promisify(parser.parseString);
            const existing = await parseString(fs.readFileSync(targetFilename));
            const unsupportedTypes = [];
            const supportedTypes = [];
            for (const types of existing.Package.types) {
                if (metadata_json_1.default.types[types['name']] != undefined)
                    if (metadata_json_1.default.types[types['name']].channels.unlockedPackagingWithoutNamespace)
                        supportedTypes.push(`${types['name']}`);
                    else
                        unsupportedTypes.push(`${types['name']}`);
            }
            if (supportedTypes.length > 0) {
                this.ux.log(`Elements supported included in your package ${packageToBeScanned['package']} are`);
                this.ux.logJson(supportedTypes);
            }
            if (unsupportedTypes.length > 0) {
                this.ux.log("Elements not supported are ");
                this.ux.logJson(unsupportedTypes);
                this.clearDirectory();
                return 1;
            }
        }
        else {
            return 2;
        }
        this.clearDirectory();
        return 0;
    }
    async clearDirectory() {
        rimraf.sync('temp_sfpowerkit');
    }
}
Valid.description = messages.getMessage('commandDescription');
Valid.examples = [
    `$ sfdx sfpowerkit:package:valid -n testPackage
  Now analyzing inspections
Converting package testPackage
Source was successfully converted to Metadata API format and written to the location: D:\projects\testPackage\temp_sfpowerkit\mdapi
Elements supported included in your package testPackage are
[
  "AuraDefinitionBundle",
  "CustomApplication",
  "ApexClass",
  "ContentAsset",
  "WorkflowRule"
]
  `
];
Valid.flagsConfig = {
    package: command_1.flags.string({ required: false, char: 'n', description: messages.getMessage('packageFlagDescription') }),
};
// Comment this out if your command does not require an org username
//protected static requiresUsername = true;
// Comment this out if your command does not support a hub org username
// protected static supportsDevhubUsername = true;
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
Valid.requiresProject = true;
exports.default = Valid;
//# sourceMappingURL=valid.js.map